/*
 *      - Sophisticated handling of debug output.
 *
 *      $Source: /usr/local/libdata/cvs/c-lib/libdebug/debug.c,v $
 *      $Revision: 1.5 $
 *      $Date: 1997/10/12 15:48:28 $
 *
 *      Copyright (C) 1996,97 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <syslog.h>
#include <stdarg.h>
#include <assert.h>

/********** Static/global variables **********/

static unsigned short DebugModules[256];

/********** Public routines **********/

/*
   Debug() is used to provide debugging information through the
   syslog(3) interface in a flexible manner. Usually you don't want to
   have all possible debug output flooded on your terminal, but you're
   interested in certain parts of the program. This can be archieved
   using _debug() or rather debug().

   Debug() expects three parameters: An id for the module your program
   is currently in, a level of how important the debug message is and
   the message itself using the usual sprintf(3) formatting.

   The module ids are integers starting at 0 and may go up to 255.
   Each module has a current debug level. You might want to take a
   look at the "debug.h-dist" include file for an idea how to define
   the appropriate labels.

   Anyway, the debug levels range from one (1) to nine (9) with one
   being the most important and nine being the least important level.
   If a module is set to debug level three (3) for example, all
   messages of the level one, two and three will be logged, while all
   messages with a level of four or higher will be ignored. A debug
   level of zero (0) will turn debugging for this module off.

   NOTE: The _debug() call should not be called directly but through the
   debug() macro as shown in "debug.h-dist".

   EXAMPLE: Here are a couple of example calls:

   debug((DEBUG_RFCPARSE, 5, "somewhat interesting information."));

   debug((3, 1, "The final result is '%s'.", string)); */

void
_debug(unsigned short module,	/* module id number */
       unsigned short level,	/* debug level from 1 to 9 */
       const char * fmt,	/* sprintf(3)-like format string */
       ...			/* arbitrary number of parameters for the sprintf(3) */
       )
{
    assert(level <= 9);
    if (level > 9)
      return;

    if (level <= DebugModules[module]) {
	va_list ap;
	va_start(ap, fmt);
	vsyslog(LOG_DEBUG, fmt, ap);
	va_end(ap);
    }
}

/*
   This function call sets the debug level for the module "module" to
   "level". A debug level of "0" will turn debug output for this
   module off.
 */

void
setDebugLevel(unsigned short module, unsigned short level)
{
    assert(level <= 9);
    if (level > 9)
      return;

    DebugModules[module] = level;
}

/*
   This function call returns the current debug level for the
   specified module.

   RETURNS: Current debug level from 0 to 9.
 */

unsigned short
getDebugLevel(unsigned short module)
{
    return DebugModules[module];
}
